#!/usr/bin/env ruby
# Disallows modifying string literals in code
# frozen_string_literal: true

# CLI Argument parsing
require 'slop'

# Web interaction libraries
require 'rubygems'
require 'mechanize'

# Instantiate an agent, and set the user-agent to spoof that of a Windows Chrome device
agent = Mechanize.new
agent.user_agent_alias = 'Windows Chrome'

# Parse options from CLI
opts = Slop.parse do |o|
  o.string '-a', '--custom-auth', 'Custom authentication mode'
  o.string 'd', 'discover', 'Set mode to discover; requires address to connect to', default: 'http://localhost'
end

# Function to check for links
def check_for_links(address, agent, indexed_pages = [])
  # Check all links on the page
  page = agent.get(address)
  if page.is_a?(Mechanize::Page)
    page.links.each do |link|
      # Get a complete resolved URI for the link
      next if link.href == 'javascript:'

      link = link.resolved_uri
      # Remove the fragment so there's less overlap
      link.fragment = nil
      # Clear query (TODO: Stash away later)
      link.query = nil

      # Check if we've seen the link before
      # Check if we're leaving the host
      if !indexed_pages.include?(link) && link.host == address.host
        # Save link to array
        indexed_pages.push(link)
      end
    end
    # Return all functions
    indexed_pages
  end
rescue Mechanize::ResponseCodeError => e
  p e.response_code
end
# opts[:custom_auth] stores the custom authentication mode
# opts[:discover] stores the address passed in at runtime


# Check for discover flag
if opts.discover?
  puts 'Mode set to Discover'
  ## If the run doesn't say it is a DVWA instance, don't do anything special
  if opts[:custom_auth] == 'dvwa'
    # From here, we can assume that we have the dvwa flag, and can act accordingly
    puts 'Set to look for DVWA-specific pages...'
    puts 'Initialising DVWA instance...'
    # Go to setup, click button
    agent.get("#{opts[:discover]}/setup.php").form(&:click_button)
    puts 'Setting security settings to low'
    # Go back to the opening page
    login_form = agent.get(opts[:discover]).form
    # Get the form from the page
    # Fill in the form with the known information
    login_form['username'] = 'admin'
    login_form['password'] = 'password'
    # `submit form
    homepage = login_form.click_button
    # Go to security page
    security_page = homepage.link(href: 'security.php').click
    # Find, modify, and submit drop-down form
    security_settings_form = security_page.forms[0]
    security_settings_form['security'] = 'low'
    updated_security_page = security_settings_form.click_button
    # Go back to home page
    updated_security_page.link(href: '.').click
  else
    puts 'No known custom auth specified'
  end
  puts '****************************'
  puts 'Discover based on common exposed pages and endpoints.... This may take some time...'
  known_uris = []
  # Check Common PHP filenames
  File.readlines('Common-PHP-Filenames.txt', chomp: true) do |file|
    file.each do |line|
      line.strip!
      begin
        agent.get(opts[:discover] + line)
        known_uris.append(URI.parse(opts[:discover] + line))
      rescue Mechanize::ResponseCodeError # => e
        # p error.response_code
      end
    end
  end
  # Brute-force using SVNDigger resources (all-extensionless + extensions from web-extensions)
  extensions = File.readlines('web-extensions.txt', chomp: true)
  filenames  = File.readlines('all-extensionless.txt', chomp: true)
  filenames.each do |filename|
    extensions.each do |extension|
      line = filename + extension
      # line.strip!
      agent.get(opts[:discover] + line)
      known_uris.append(URI.parse(opts[:discover] + line))
    rescue Mechanize::ResponseCodeError # => e
      # p error.response_code
    end
  end
  puts 'Guessed URIs: '
  known_uris.each do |uri|
    p uri
  end
  puts '****************************'
  # Link crawling.....
  crawl_starting_address = opts[:discover]
  if opts[:custom_auth] == 'dvwa'
    page = agent.get(opts[:discover])
    login_form = page.form
    login_form['username'] = 'admin'
    login_form['password'] = 'password'
    # submit form
    homepage = login_form.click_button
    crawl_starting_address = homepage.uri
  end

  addresses = check_for_links(Uri.parse(crawl_starting_address), agent)
  old_addresses = []
  until addresses - old_addresses == [] && old_addresses - addresses == []
    old_addresses = Array.new(addresses)
    addresses.each do |address|
      appending_addrs = check_for_links(address, agent, addresses)
      addresses = addresses.union(appending_addrs) unless appending_addrs.nil?
    end
  end

  puts 'Crawled URIs: '
  final_filter = addresses.union(old_addresses)
  final_filter.each do |link|
    p link
  end
  # Look for URL parameters
  # Look for form parameters
  # look for cookies
end
